########################################################################
# sbvar.py - common global variable declarations and default values
#
#    Copyright (C) 2015-2017 Anders Lind, Erding Mallards e.V.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Global variable initialization module
# Defines all global variables with default values
# Must be imported by all modules dealing with any of the variables
# 
# Each variable is then pre-pended with "sbvar". Example:
# Global variable "counter_G1" is referenced as "sbvar.counter_G1".
#
# Version 3, April 14, 2017

########################################################################
# Debug mode switches:
debug_mode = 0
emulation_mode = 0

########################################################################
# Counters - holding the current game results
# Guest inning counters:
counter_G1 = 0
counter_G2 = 0
counter_G3 = 0
counter_G4 = 0
counter_G5 = 0
counter_G6 = 0
counter_G7 = 0
counter_G8 = 0
counter_G9 = 0
counter_GX = 0
# Home inning counters:
counter_H1 = 0
counter_H2 = 0
counter_H3 = 0
counter_H4 = 0
counter_H5 = 0
counter_H6 = 0
counter_H7 = 0
counter_H8 = 0
counter_H9 = 0
counter_HX = 0
# Error counters:
counter_GE = 0
counter_HE = 0
# Runs counters:
counter_GR = 0
counter_HR = 0
# Hits counters: 
counter_GH = 0
counter_HH = 0
# Outs, balls, strikes counters:
counter_OUTS = 0
counter_BALLS = 0
counter_STRIKES = 0

########################################################################
# Visibility flags - 0 = digit is hidden, 1 = digit is displayed
# Guest inning flags:
show_G1 = 0
show_G2 = 0
show_G3 = 0
show_G4 = 0
show_G5 = 0
show_G6 = 0
show_G7 = 0
show_G8 = 0
show_G9 = 0
show_GX = 0
# Home inning flags:
show_H1 = 0
show_H2 = 0
show_H3 = 0
show_H4 = 0
show_H5 = 0
show_H6 = 0
show_H7 = 0
show_H8 = 0
show_H9 = 0
show_HX = 0
# Error flags:
show_GE = 0
show_HE = 0
# Runs flags:
show_GR = 0
show_HR = 0
# Hits flags:
show_GH = 0
show_HH = 0
# Outs, balls, strikes counters:
show_OUTS = 0
show_BALLS = 0
show_STRIKES = 0
# Colon (when displaying the time)
show_COLON = 0

########################################################################
# Current inning counter:
counter_INNING = 0

########################################################################
# Current display mode:
# 0 = Time mode
# 1 = Baseball mode
# 2 = Teeball mode
# 3 = Startup message
# 4 = Homerun animation
display_mode = 3

########################################################################
# Debug color codes:
col_fs = '\033[1;37m' # Bright white
col_fe = '\033[1;33m' # Bright yellow
col_wa = '\033[1;35m' # Bright cyan
col_er = '\033[1;31m' # Bright red inverted
col_n  = '\033[0;37m' # Normal white
col_t  = '\033[2;32m' # Faint green

########################################################################
# Mapping of ASCII characters to display hex values
char_code = dict({ \
    'A': 0x77, \
    'B': 0x7F, \
    'C': 0x39, \
    'D': 0x5E, \
    'E': 0x79, \
    'F': 0x71, \
    'G': 0x3D, \
    'H': 0x76, \
    'I': 0x06, \
    'J': 0x1E, \
    'K': 0x75, \
    'L': 0x38, \
    'M': 0xA7, \
    'N': 0xE4, \
    'O': 0x3F, \
    'P': 0x73, \
    'Q': 0x67, \
    'R': 0xD3, \
    'S': 0x6D, \
    'T': 0x78, \
    'U': 0x3E, \
    'V': 0xD2, \
    'W': 0x9E, \
    'X': 0xC6, \
    'Y': 0x6E, \
    'Z': 0x5B, \
    'Ä': 0x59, \
    'Ö': 0x5D, \
    'Ü': 0x1D, \
    'ß': 0xCF, \
    'a': 0x5F, \
    'b': 0x7C, \
    'c': 0x58, \
    'd': 0x5E, \
    'e': 0x7B, \
    'f': 0x71, \
    'g': 0x6F, \
    'h': 0x74, \
    'i': 0x04, \
    'j': 0x1E, \
    'k': 0x75, \
    'l': 0x38, \
    'm': 0xA7, \
    'n': 0x54, \
    'o': 0x5C, \
    'p': 0x73, \
    'q': 0x67, \
    'r': 0x50, \
    's': 0x6D, \
    't': 0x78, \
    'u': 0x1C, \
    'v': 0xD2, \
    'w': 0x9E, \
    'x': 0xC6, \
    'y': 0x6E, \
    'z': 0x5B, \
    'ä': 0x59, \
    'ö': 0x5D, \
    'ü': 0x1D, \
    '-': 0x40, \
    '_': 0x08, \
    ' ': 0x00, \
    '0': 0x3F, \
    '1': 0x06, \
    '2': 0x5B, \
    '3': 0x4F, \
    '4': 0x66, \
    '5': 0x6D, \
    '6': 0x7D, \
    '7': 0x07, \
    '8': 0x7F, \
    '9': 0x6F, \
    '°': 0x63 \
    })

########################################################################
# Variables holding HOMERUN animation sequences (as lists):
homerun_G1 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, \
0x38, 0x0E, 0x0, 0x31, 0x07, 0x0, 0x31, 0x87, 0x0, 0x39, \
0x09, 0x0, 0x39, 0x0F, 0x0, 0x30, 0x06, 0x0, 0x37, 0x06, 0x0]
homerun_G2 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x38, \
0x0E, 0x0, 0x31, 0x07, 0x0, 0x31, 0x87, 0x0, 0x39, 0x09, \
0x0, 0x39, 0x0F, 0x0, 0x30, 0x06, 0x0, 0x37, 0x06, 0x0, 0x0]
homerun_G3 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x38, 0x0E, \
0x0, 0x31, 0x07, 0x0, 0x31, 0x87, 0x0, 0x39, 0x09, 0x0, \
0x39, 0x0F, 0x0, 0x30, 0x06, 0x0, 0x37, 0x06, 0x0, 0x0, 0x0]
homerun_G4 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x38, 0x0E, 0x0, \
0x31, 0x07, 0x0, 0x31, 0x87, 0x0, 0x39, 0x09, 0x0, 0x39, \
0x0F, 0x0, 0x30, 0x06, 0x0, 0x37, 0x06, 0x0, 0x0, 0x0, 0x0]
homerun_G5 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x38, 0x0E, 0x0, 0x31, \
0x07, 0x0, 0x31, 0x87, 0x0, 0x39, 0x09, 0x0, 0x39, 0x0F, \
0x0, 0x30, 0x06, 0x0, 0x37, 0x06, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_G6 = [0x0, 0x0, 0x0, 0x0, 0x38, 0x0E, 0x0, 0x31, 0x07, \
0x0, 0x31, 0x87, 0x0, 0x39, 0x09, 0x0, 0x39, 0x0F, 0x0, \
0x30, 0x06, 0x0, 0x37, 0x06, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_G7 = [0x0, 0x0, 0x0, 0x38, 0x0E, 0x0, 0x31, 0x07, 0x0, \
0x31, 0x87, 0x0, 0x39, 0x09, 0x0, 0x39, 0x0F, 0x0, 0x30, \
0x06, 0x0, 0x37, 0x06, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_G8 = [0x0, 0x0, 0x38, 0x0E, 0x0, 0x31, 0x07, 0x0, 0x31, \
0x87, 0x0, 0x39, 0x09, 0x0, 0x39, 0x0F, 0x0, 0x30, 0x06, \
0x0, 0x37, 0x06, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_G9 = [0x0, 0x38, 0x0E, 0x0, 0x31, 0x07, 0x0, 0x31, \
0x87, 0x0, 0x39, 0x09, 0x0, 0x39, 0x0F, 0x0, 0x30, 0x06, \
0x0, 0x37, 0x06, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_GX = [0x38, 0x0E, 0x0, 0x31, 0x07, 0x0, 0x31, 0x87, \
0x0, 0x39, 0x09, 0x0, 0x39, 0x0F, 0x0, 0x30, 0x06, 0x0, \
0x37, 0x06, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_H1 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, \
0x30, 0x06, 0x0, 0x38, 0x0E, 0x0, 0x30, 0x06, 0x0, 0x38, \
0x08, 0x0, 0x30, 0x64, 0x0, 0x38, 0x0E, 0x0, 0x30, 0x3E, 0x0]
homerun_H2 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30, \
0x06, 0x0, 0x38, 0x0E, 0x0, 0x30, 0x06, 0x0, 0x38, 0x08, \
0x0, 0x30, 0x64, 0x0, 0x38, 0x0E, 0x0, 0x30, 0x3E, 0x0, 0x0]
homerun_H3 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30, 0x06, \
0x0, 0x38, 0x0E, 0x0, 0x30, 0x06, 0x0, 0x38, 0x08, 0x0, \
0x30, 0x64, 0x0, 0x38, 0x0E, 0x0, 0x30, 0x3E, 0x0, 0x0, 0x0]
homerun_H4 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30, 0x06, 0x0, \
0x38, 0x0E, 0x0, 0x30, 0x06, 0x0, 0x38, 0x08, 0x0, 0x30, \
0x64, 0x0, 0x38, 0x0E, 0x0, 0x30, 0x3E, 0x0, 0x0, 0x0, 0x0]
homerun_H5 = [0x0, 0x0, 0x0, 0x0, 0x0, 0x30, 0x06, 0x0, 0x38, \
0x0E, 0x0, 0x30, 0x06, 0x0, 0x38, 0x08, 0x0, 0x30, 0x64, \
0x0, 0x38, 0x0E, 0x0, 0x30, 0x3E, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_H6 = [0x0, 0x0, 0x0, 0x0, 0x30, 0x06, 0x0, 0x38, 0x0E, \
0x0, 0x30, 0x06, 0x0, 0x38, 0x08, 0x0, 0x30, 0x64, 0x0, \
0x38, 0x0E, 0x0, 0x30, 0x3E, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_H7 = [0x0, 0x0, 0x0, 0x30, 0x06, 0x0, 0x38, 0x0E, 0x0, \
0x30, 0x06, 0x0, 0x38, 0x08, 0x0, 0x30, 0x64, 0x0, 0x38, \
0x0E, 0x0, 0x30, 0x3E, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_H8 = [0x0, 0x0, 0x30, 0x06, 0x0, 0x38, 0x0E, 0x0, 0x30, \
0x06, 0x0, 0x38, 0x08, 0x0, 0x30, 0x64, 0x0, 0x38, 0x0E, \
0x0, 0x30, 0x3E, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_H9 = [0x0, 0x30, 0x06, 0x0, 0x38, 0x0E, 0x0, 0x30, 0x06, \
0x0, 0x38, 0x08, 0x0, 0x30, 0x64, 0x0, 0x38, 0x0E, 0x0, \
0x30, 0x3E, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
homerun_HX = [0x30, 0x06, 0x0, 0x38, 0x0E, 0x0, 0x30, 0x06, 0x0, \
0x38, 0x08, 0x0, 0x30, 0x64, 0x0, 0x38, 0x0E, 0x0, 0x30, \
0x3E, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]

########################################################################
# Variables holding the player information:
player_number = dict(zip(\
    ['G1', 'G2', 'G3', 'G4', 'G5', 'G6', 'G7', 'G8', 'G9', \
    'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'H7', 'H8', 'H9'], \
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]))
player_position = dict(zip(\
    ['G1', 'G2', 'G3', 'G4', 'G5', 'G6', 'G7', 'G8', 'G9', \
    'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'H7', 'H8', 'H9'], \
    ['', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '']))
player_firstname = dict(zip(\
    ['G1', 'G2', 'G3', 'G4', 'G5', 'G6', 'G7', 'G8', 'G9', \
    'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'H7', 'H8', 'H9'], \
    ['', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '']))
player_lastname = dict(zip(\
    ['G1', 'G2', 'G3', 'G4', 'G5', 'G6', 'G7', 'G8', 'G9', \
    'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'H7', 'H8', 'H9'], \
    ['', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '']))
player_hits = dict(zip(\
    ['G1', 'G2', 'G3', 'G4', 'G5', 'G6', 'G7', 'G8', 'G9', \
    'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'H7', 'H8', 'H9'], \
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]))
player_atbats = dict(zip(\
    ['G1', 'G2', 'G3', 'G4', 'G5', 'G6', 'G7', 'G8', 'G9', \
    'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'H7', 'H8', 'H9'], \
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]))
current_batter_guest = 9
current_batter_home = 9

# END OF FILE